---
title: "Sesión 1 - Presentando Datos"
author: "Álvaro López Z"
date: "2024-10-14"
output: html_document
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```
## Introducción
En este primer módulo se revisaran temas relacionados con la presentación de datos utilizando R.

Los temas a revisar son:

- Tablas

  - Función `aggregate()`
  - Función `groupby()`
  - Función `apply()` y sus variantes
  
- Histogramas

  - Función `hist()` y sus componentes
  - Función `lines()`
  
- Box Plot
  
- Scatter Plots

  - Función `plot()`
  - Función `pairs()`


## Tablas

```{r cars, comment=""}
# Utilizaremos los datos mtcars
data("mtcars")
print(mtcars)
```

```{r, comment=""}
# Tabla básica: Cuenta las ocurrencias de cada valor único en un vector
# Esto muestra cuántos coches tienen 4, 6 u 8 cilindros
table(mtcars$cyl)
```

```{r, comment=""}
# Tabla de dos vías: Muestra la relación entre dos variables categóricas
# Esto muestra cuántos coches tienen cada combinación de cilindros y velocidades
table(mtcars$cyl, mtcars$gear)
```

```{r, comment=""}
# Tabla de contingencia: Otra forma de crear una tabla de dos vías
# xtabs() es más flexible y puede manejar fórmulas
tabla_cont <- xtabs(~ cyl + gear, data = mtcars)
print(tabla_cont)
```

```{r, comment=""}
# Tabla de proporciones: Muestra la proporción de cada celda en la tabla de contingencia
# Esto es útil para entender las frecuencias relativas
prop.table(tabla_cont)

# Comprobemos que las proporciones suman 1
sum(prop.table(tabla_cont))
```

```{r, message=F}
# Ejercicio 1: Crear una tabla que muestre la proporción de coches con cada tipo de transmisión (am) para cada número de cilindros (cyl)

```

### Group by
```{r, message=F, comment=""}
# Usando aggregate() para operaciones de agrupación
# Calcular la media de mpg para cada grupo de cilindros
aggregate(mpg ~ cyl, data = mtcars, FUN = mean)
```

```{r, comment=""}
# Múltiples variables de agrupación
# Calcular la mediana de mpg y caballos de fuerza para cada combinación de cilindros y tipo de transmisión
aggregate(cbind(mpg, hp) ~ cyl + am, data = mtcars, FUN = mean)
```

```{r, comment=""}
# Usando by() para operaciones más complejas
# Calcular la correlación entre mpg y peso para cada grupo de cilindros
by(mtcars, mtcars$cyl, function(subconjunto) cor(subconjunto$mpg, subconjunto$wt))
```

```{r, message=F, comment=""}
# Cargar la biblioteca dplyr
library(dplyr)

# Usando group_by() y summarise() de dplyr
# Calcular la media de mpg para cada grupo de cilindros
mtcars %>%
  group_by(cyl) %>%
  summarise(mpg_promedio = mean(mpg))
```

```{r, comment="", message=F}
# Múltiples variables de agrupación y múltiples resúmenes con dplyr
mtcars %>%
  group_by(cyl, am) %>%
  summarise(
    mpg_promedio = mean(mpg),
    hp_promedio = mean(hp),
    n = n()
  )
```
```{r, message=F, comment=""}
# Operaciones más complejas con group_by() y mutate()
mtcars %>%
  group_by(cyl) %>%
  mutate(
    mpg_centrado = mpg - mean(mpg),
    mpg_z_score = (mpg - mean(mpg)) / sd(mpg)
  ) %>%
  select(mpg, cyl, mpg_centrado, mpg_z_score)
```

```{r, message=F}
# Ejercicio 2: Usar una operación de agrupación para calcular la potencia máxima (hp) para cada combinación de cilindros y tipo de transmisión
```


### Apply
```{r, comment=""}
# apply(): Se usa para aplicar una función a filas o columnas de una matriz
# Calcular la media para cada columna
apply(mtcars, 2, mean)  # 2 indica que estamos aplicando la función a las columnas
```

```{r, comment=""}
# lapply(): Aplica una función a cada elemento de una lista
# Aplicar la función logaritmo a cada columna
lapply(mtcars, log)

# sapply(): Una versión más amigable de lapply() que simplifica la salida
# Obtener la clase de cada columna
sapply(mtcars, log)

# mapply(): Aplica una función a múltiples listas o vectores en paralelo
# Combinar columnas con una función personalizada
mapply(function(x, y) x * y, mtcars$hp, mtcars$cyl)

# tapply(): Aplica una función agrupando por las variables indicadas
# Calcular la mediana de mpg para cada grupo de cilindros
tapply(mtcars$mpg, mtcars$cyl, median)
```

```{r, comment=""}
# Ejercicio 3: Usar una función apply para calcular la desviación estándar de cada columna numérica en mtcars
```


## Histogramas
```{r, comment=""}
hist(mtcars$mpg, 
     breaks = 10,        # Número de barras
     col = "skyblue",    # Color de las barras
     border = "white",   # Color de los bordes de las barras
     main = "Histograma de MPG",
     xlab = "Millas por Galón",
     ylab = "Frecuencia")
```

```{r, comment=""}
# Histograma con curva de densidad
hist(mtcars$mpg, 
     freq = FALSE,      # Graficar densidad en lugar de frecuencia
     breaks = 10,
     col = "lightgreen",
     main = "Histograma con Curva de Densidad",
     xlab = "Millas por Galón")
lines(density(mtcars$mpg), col = "red", lwd = 2)
```

```{r, comment=""}
# Que pasa si guardamos el histograma en una variable?
histograma <- hist(mtcars$mpg, 
     breaks = 10,        # Número de barras
     col = "skyblue",    # Color de las barras
     border = "white",   # Color de los bordes de las barras
     main = "Histograma de MPG",
     xlab = "Millas por Galón",
     ylab = "Frecuencia")

histograma
```
## Box Plot
```{r, comment=""}
boxplot(mtcars$wt, border = "darkred", col ="orange", 
      main = "WT",
      cex.main = 1.5, horizontal = T)
```


## Scatter Plots
```{r, comment=""}
# Gráfico de Dispersión Simple
plot(mtcars$wt, mtcars$mpg, 
     main = "Peso vs MPG",
     xlab = "Peso",
     ylab = "Millas por Galón",
     pch = 19,         # Carácter del punto (círculos llenos)
     col = "blue")
```

```{r, comment=""}
# Gráfico de Dispersión Pareado
pairs(mtcars[, c("mpg", "disp", "hp", "wt")],
      main = "Gráficos de Dispersión Pareados",
      pch = 21,
      bg = c("red", "green", "blue")[unclass(mtcars$cyl)])
```

## Bonus
Creemos nuestra propia función para hace una gráfica de dispersión a pares
```{r, message=FALSE, comment="", warning=FALSE}
# Primero, creemos una función que obtendrá la distribución de las variables
panel.hist <- function(x, ...)
  {
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(usr[1:2], 0, 1.5) )
    h <- hist(x, plot = FALSE)
    breaks <- h$breaks; nB <- length(breaks)
    y <- h$counts; y <- y/max(y)
    rect(breaks[-nB], 0, breaks[-1], y, col = "gray",...)
  }

# Ahora, escribamos una función que calcule las correlaciones
panel.cor <- function(x, y, digits = 2, cex.cor = 1.5, ...)
  {
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(0, 1, 0, 1))
    r <- cor(x, y)
    txt <- format(r, digits = digits)
    text(0.5, 0.5, txt, cex = cex.cor)
}

# Veamos nuestra nueva gráfica
pairs(mtcars[, c("mpg", "disp", "hp", "wt")], main = "Matriz de dispersion", labels = c("mpg", "disp", "hp", "wt"), cex.labels = 0.9, upper.panel = panel.cor, diag.panel = panel.hist)
  
```

